	list p=12F675, b=8, c= 102, n=71, t=on, st=off, f=inhx32
;******************************************************************
;*								  *
;*  Filename: 12F675 DS18B20 v2.asm                               *
;*    Author: Mike McLaren, K8LH   (k8lh@arrl.net)                *
;*      Date: 12-Sep-06 (last revised 14-Sep-06 @ 1400)           *
;*                                                                *
;*  Dallas DS18B20 Temperature Sensor and Serial I/O Demo         *
;*                                                                *
;*     MPLab: 7.40    (tabs=8)                                    *
;*     MPAsm: 5.03                                                *
;*                                                                *
;******************************************************************
;*                                                                *
;*   Half Duplex Bit-Banged 9600 Baud Serial I/O Demo             *
;*    (2-Pin Version with 8-Byte Circular RX Buffer)              *
;*                                                                *
;*  Uses 12F675 INTOSC running at 4-MHz                          *
;*  Bit rate error 0.16% plus or minus 1.0% for INTOSC           *
;*  Bit-banged 9600 baud serial I/O                              *
;*    Half Duplex (should not TX and RX simultaneously)          *
;*    TMR0 interrupts at 104-usec intervals (every 104           *
;*     instruction cycles) and IOC (interrupt on change)          *
;*     for RX start bit leading edge detection on RXPIN           *
;*    Circular 8-byte RX character buffer                        *
;*    Inverted TX and RX signals (MAX232A or similar             *
;*     inverting RS-232 interface required)                       *
;*  Relatively small - the ISR and the support routines          *
;*   Init232, Put232, Get232, and PutString use 124 words         *
;*   of code space at locations 0004 through 007F and 20          *
;*   of 64 RAM locations for variables and RX buffer.             *
;*  Worst case 51% ISR instruction cycle 'overhead'              *
;*   (54-usecs) when a complete RX character is added to          *
;*   the circular buffer once every 1.04-msecs.                   *
;*                                                                *
;******************************************************************

        #include <p12f675.inc>
        errorlevel -302

        __config _MCLRE_OFF & _WDT_OFF & _INTRC_OSC_NOCLKOUT

;
;  RS-232 file register variables, 16 locations [20..2F]
;
RXBUFF  equ     h'20'           ; RX buffer, 8 bytes [20..27]
;
PROC232 equ     h'28'           ; RS-232 Process Latch
TXCNT   equ     h'29'           ; TX-232 bit count
TXVAR   equ     h'2A'           ; TX-232 data byte
RXCNT   equ     h'2B'           ; RX-232 bit count
RD_PTR  equ     h'2C'           ; RX buffer RD pointer
WR_PTR  equ     h'2D'           ; RX buffer WR pointer
RXCHAR  equ     h'2E'           ; RX char buffer for main
FSRTMP  equ     h'2F'           ;
;
;  other variables
;
PTRL    equ     h'30'           ; PutString routine
PTRH    equ     h'31'           ; PutString routine
TEMP    equ     h'32'           ; PutByte routine
;
;  PROC232 flag bits
;
RXFLAG  equ     0x00            ; 1 = RX in progress
TXFLAG  equ     0x02            ; 1 = TX in progress
;
;  RS-232 constants
;
TXPIN   equ     h'05'           ; TXPIN = GP5 (pin 2) on RGBX board
RXPIN   equ     h'03'           ; RXPIN = GP3 (pin 4)
;
;  DS18B20 variables
;
PDByte  equ     h'33'           ;
Count   equ     h'34'           ;
IOByte  equ     h'35'           ;
TempL   equ     h'36'           ;
TempH   equ     h'37'           ;
;
;  DS1820 constants
;
DQ      equ     h'02'           ; GP2 (pin 05)
;
;  Mult8x16 routine
;
Mult8   equ     0x38            ; multiplier
MultL   equ     0x39            ; multiplicand lo
MultH   equ     0x3A            ; multiplicand hi
ProdL   equ     0x3B            ; product lo
ProdH   equ     0x3C            ; product hi
ProdU   equ     0x3D            ; product ul
;
;  Bin2Dec routine
;
D0      equ     0x40            ; ones
D1      equ     0x41            ; tens
D2      equ     0x42            ; hundreds
D3      equ     0x43            ; thousands
;
; file locations used by ISR for saving & restoring context
;
W_ISR   equ     h'5C'           ; ISR 'W'
S_ISR   equ     h'5D'           ; ISR 'STATUS'
P_ISR   equ     h'5E'           ; ISR 'PCLATH'
F_ISR   equ     h'5F'           ; ISR 'FSR'

;******************************************************************
;
;  Title macro - home cursor, clear screen, print a string
;
_Title  macro   str             ;
        local   String, P
        movlw   low String      ;
        movwf   PTRL            ;
        movlw   high String     ;
        movwf   PTRH            ;
        goto    P               ;
String  dt      0x1b,"[2J"      ; home cursor, clear screen
        dt      str,0
P       call    PutString       ; print string
        endm
;
;  Print macro - print in-line character string
;
Print   macro   str             ;
        local   String, P
        movlw   low String      ;
        movwf   PTRL            ;
        movlw   high String     ;
        movwf   PTRH            ;
        goto    P               ;
String  dt      str,0
P       call    PutString       ; print string
        endm

;******************************************************************
;*								  *
;*								  *
;*								  *
;*								  *
;*								  *
;******************************************************************

	org	0x0000

RESET   clrf    STATUS          ;                                 |B0
        movlw   b'00000111'     ;                                 |B0
        movwf   CMCON           ; comparator off > digital I/O    |B0
        goto    Main            ;                                 |B0

;******************************************************************
;*                                                             	  *
;*    Interrupt Service Routine	for 'bit-banged' Serial I/O       *
;*                                                                *
;*    Interrupts are generated every 104-us by TMR0 for 9600      *
;*    baud shift operations and by IOC (interrupt on change)      *
;*    on the start bit leading edge on RXPIN (GP3, pin 4).        *
;*                                                                *
;*    On detecting the RX start bit leading edge, GP3 IOC is      *
;*    turned off, RXFLAG is set to indicate rx-in-progress,       *
;*    RXCNT bit count variable is set, and TMR0 is set up to      *
;*    interrupt in approximately 1/2 bit time (52-usec).          *
;*                                                                *
;******************************************************************

        org     0x0004
;
;  save context
;
ISR     movwf   W_ISR           ; save W-reg                      |B?
        swapf   STATUS,W        ; doesn't change STATUS bits      |B?
	movwf	S_ISR		; save STATUS reg		  |B?
	clrf	STATUS		; bank 0			  |B0
	movf	PCLATH,W	; get PCLATH			  |B0
	movwf	P_ISR		; save PCLATH			  |B0
	movf	FSR,W		; get FSR			  |B0
	movwf	F_ISR		; save FSR			  |B0
;
;  test for GP3 (RXPIN) start bit leading edge
;
ISR_IOC	btfss	INTCON,GPIF	; IOC "start bit" interrupt?	  |B0
	goto	ISR_SET		; no, branch			  |B0
	movf	GPIO,W		; yes, read GPIO		  |B0
	bcf	INTCON,GPIF	; clear IOC interrupt flag	  |B0
	btfsc	GPIO,RXPIN	; RXPIN low (start bit)?	  |B0
	goto	ISR_XIT		; no, wrong polarity, branch	  |B0
	bsf	PROC232,RXFLAG	; indicate RX in progress	  |B0
	movlw	d'10'		; (1 start + 8 data + 1 stop)	  |B0
	movwf	RXCNT		; initialize RX bit counter	  |B0
;
	bcf	INTCON,T0IF	; clear TMR0 interrupt flag	  |B0
        movlw   -d'52'+d'39'    ; 1/2 bit time (52-usecs)         |B0
	movwf	TMR0		;				  |B0
	goto	ISR_X		; turn off GP3 IOC & exit	  |B0
;
;  clear TMR0 interrupt flag, prep TMR0 for next interrupt
;
ISR_SET bcf     INTCON,T0IF     ; clear TMR0 interrupt flag       |B0
        movlw   -d'104'+2       ; setup for next 104-usec int     |B0
        addwf   TMR0,f          ;                                 |B0
;
;  bit-banged serial output
;
ISR_TX	btfss	PROC232,TXFLAG	; TX in progress?		  |B0
	goto	ISR_RX		; no, branch			  |B0
	movf	TXCNT,W		; get TX bit count		  |B0
	bnz	ISR_TX0		; no, branch, send next bit	  |B0
	movlw	d'10'		; 10 bits (start + 8 data + stop) |B0
	movwf	TXCNT		; initialize TX bit counter	  |B0
	goto	ISR_TX1		; send the start bit		  |B0
ISR_TX0	rrf	TXVAR,f		; shift lsb into C		  |B0
	bsf	TXVAR,7		; set bit 7 for stop bits	  |B0
	btfsc	STATUS,C	; skip if C=0			  |B0
	bsf	GPIO,TXPIN	; set TX pin			  |B0
	btfss	STATUS,C	; skip if C=1			  |B0
ISR_TX1	bcf	GPIO,TXPIN	; clr TX pin			  |B0
	decfsz	TXCNT,f		; last bit shifted out?		  |B0
	goto	ISR_RX		; no, branch			  |B0
	bcf	PROC232,TXFLAG	; yes, indicate end of TX	  |B0
;
;  bit-banged serial input
;
ISR_RX	btfss	PROC232,RXFLAG	; RX in progress?		  |B0
	goto	ISR_XIT		; no, branch, else		  |B0
	movf	WR_PTR,W	; get WR_PTR (RXBUFF + 00..07)	  |B0
	movwf	FSR		; setup indirect address	  |B0
	bcf	STATUS,C	; clear Carry, assume data = 0	  |B0
	btfsc	GPIO,RXPIN	; TRPIN = 0 ?			  |B0
	bsf	STATUS,C	; no, set Carry, data = 1	  |B0
	rrf	INDF,f		; shift bit into our character	  |B0
	decfsz	RXCNT,f		; all 10 bits?			  |B0
	goto	ISR_XIT		; no, branch			  |B0
	rlf	INDF,f		; yes, get rid of the stop bit	  |B0
;
;  we now have a complete RX character in RXBUFF[WR_PTR]
;
	bcf	PROC232,RXFLAG	; clear RX-in-progress flag	  |B0
;
;  if WR_PTR + 1 = RD_PTR the buffer is full so exit, else
;  increment WR_PTR variable for the next receive character
;
	incf	WR_PTR,W	; W = WR_PTR + 1		  |B0
	andlw	RXBUFF+h'07'	; this works for 20..27		  |B0
	xorwf	RD_PTR,W	; WR_PTR+1 = RD_PTR (full)?	  |B0
	skpz			; yes, ignore new character	  |B0
	incf	WR_PTR,f	; else add it to the buffer	  |B0
	bcf	WR_PTR,3	; keep in range 20..27		  |B0
;
;  when entered from the ISR_IOC routine, this code turns off
;  the interrupt-on-change feature while receiving a character
;
;  when fallen into from the ISR_RX routine above, this code
;  turns interrupt-on-change back on in order to detect the
;  next receive character start bit falling edge
;
ISR_X	bsf	STATUS,RP0	; select bank 1			  |B1
	movlw	b'00001000'	; mask for RXPIN/GP3 IOC	  |B1
	xorwf	IOC,f		; toggle RXPIN/GP3 IOC		  |B1
	bcf	STATUS,RP0	; select bank 0			  |B0
;
;  restore context
;
ISR_XIT	movf	F_ISR,W		;				  |B0
	movwf	FSR		; restore FSR			  |B0
	movf	P_ISR,W		;				  |B0
	movwf	PCLATH		; restore PCLATH		  |B0
	swapf	S_ISR,W		;				  |B0
	movwf	STATUS		; restore STATUS		  |B?
	swapf	W_ISR,f		; don't screw up STATUS		  |B?
	swapf	W_ISR,W		; restore W-reg			  |B?
	retfie			; return from interrupt		  |B?

;******************************************************************
;*								  *
;*  Companion Put232 and Get232 subroutines			  *
;*								  *
;******************************************************************
;
;  Put232 - enter with character to be sent in W
;
Put232	btfsc	PROC232,TXFLAG	; TX in progress?		  |B0
	goto	Put232		; yes, branch and wait		  |B0
	movwf	TXVAR		; stuff character		  |B0
	bsf	PROC232,TXFLAG	; initiate TX			  |B0
	return			;				  |B0
;
;  Get232 - exit with received character in W
;	  - performs RXBUFF 'unload buffer' operation
;
Get232	movf	RD_PTR,W	;				  |B0
	xorwf	WR_PTR,W	; RD_PTR = WR_PTR (buff empty)?	  |B0
	bz	Get232		; yes, loop, wait for character	  |B0
	movf	FSR,W		; get and save FSR		  |B0
	movwf	FSRTMP		;				  |B0
	movf	RD_PTR,W	;				  |B0
	movwf	FSR		; setup indirect address	  |B0
	movf	INDF,W		; get RXBUFF[RD_PTR] character	  |B0
	movwf	RXCHAR		; save it for later		  |B0
	movf	FSRTMP,W	; 				  |B0
	movwf	FSR		; restore FSR			  |B0
	incf	RD_PTR,W	; increment RD_PTR		  |B0
	andlw	b'00100111'	; keep it in range 20..27	  |B0
	movwf	RD_PTR		;				  |B0
	movf	RXCHAR,W	; get receive character		  |B0
	return			;				  |B0

;******************************************************************
;*								  *
;*  Companion Init232 subroutine				  *
;*								  *
;******************************************************************

Init232	movlw	RXBUFF		; RX circular buffer address	  |B0
	movwf	RD_PTR		; init circular buffer RD ptr	  |B0
	movwf	WR_PTR		; init circular buffer WR ptr	  |B0
	clrf	PROC232		; clear RS232 process latch	  |B0
	clrf	TXCNT		; init TXCNT variable		  |B0
        bsf     GPIO,TXPIN      ; put TXPIN in idle state         |B0
;
	bsf	STATUS,RP0	; bank 1			  |B1
	bsf	TRISIO,RXPIN	; set RXPIN as input    	  |B1
	bcf	TRISIO,TXPIN	; set TXPINas output    	  |B1
	movlw	b'10001000'	; no weak pull-ups and no	  |B1
	movwf	OPTION_REG	; prescaler for TMR0		  |B1
	bsf	IOC,RXPIN	; turn on RXPIN IOC		  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
;
	clrf	INTCON		; clear interrupt flags/enables	  |B0
	bsf	INTCON,T0IE	; enable TMR0 interrupts	  |B0
	bsf	INTCON,GPIE	; enable IOC interrupts		  |B0
	bsf	INTCON,GIE	; enable global interrupts	  |B0
	return			;				  |B0

;******************************************************************
;
;  PutString sends a character string through the RS232 port
;
;  Entry: setup PTRL and PTRH to string address before entry
;	  string must be terminated with a 00 byte
;
PutString
	call	GetTable	; get a table character		  |B0
	andlw	b'11111111'	;				  |B0
	skpnz			; 00 byte, last character?	  |B0
	return			; yes, return			  |B0
	call	Put232		; else,output character		  |B0
	incfsz	PTRL,F		; increment pointer		  |B0
	goto	PutString	;				  |B0
	incf	PTRH,F		;				  |B0
	goto	PutString	;				  |B0
;
GetTable
	movf	PTRH,W		;				  |B0
	movwf	PCLATH		;				  |B0
	movf	PTRL,W		;				  |B0
	movwf	PCL		;				  |B0
;
;  Print byte in W as two ASCII nybbles
;
PutByte movwf   TEMP            ; save byte                       |B0
        swapf   TEMP,W          ; swap nybbles in W               |B0
        call    Hex2Asc         ; process left nybble             |B0
        movf    TEMP,W          ; process right nybble            |B0
Hex2Asc andlw   b'00001111'     ; mask off left nybble            |B0
        addlw   a'6'            ; 0-9>36-3F, A-F>40-45            |B0
        btfsc   STATUS,DC       ; A..F?  no, skip, else           |B0
        addlw   h'07'           ; A-F (40-45) > 47-4C             |B0
        addlw   0-6             ; '0'..'9' or 'A'..'F'            |B0
        goto    Put232          ; print ASCII nybble              |B0

;******************************************************************
;*                                                                *
;*  Dallas/Maxim DS18B20 Temperature Sensor Routines              *
;*                                                                *
;******************************************************************

OW_HIZ  MACRO
        bsf     STATUS,RP0      ; bank 1                          |B1
        bsf     TRISIO,DQ       ; set DQ pin as HiZ input         |B1
        bcf     STATUS,RP0      ; bank 0                          |B0
        ENDM                    ;                                 |B0

OW_LO   MACRO
        bcf     GPIO,DQ         ; set output latch lo             |B0
        bsf     STATUS,RP0      ; bank 1                          |B1
        bcf     TRISIO,DQ       ; set DQ pin to an output         |B1
        bcf     STATUS,RP0      ; bank 0                          |B0
        ENDM                    ;                                 |B0

WAIT    MACRO   TIME            ;                                 |B0
        movlw   (TIME/5)-1      ; 1 sec                          |B0
        call    WAIT5U          ; 2 sec                          |B0
        ENDM

WAIT5U  nop                     ; 1 usec                          |B0
        addlw   -1              ; 1 usec                          |B0
        bnz     WAIT5U          ; 3 usec or 2 usec                |B0
        return                  ; 2 usec                          |B0

OW_RESET
        OW_HIZ                  ;                                 |B0
        clrf    PDByte          ;                                 |B0
        OW_LO                   ;                                 |B0
        WAIT    .500            ;                                 |B0
        bcf     INTCON,GIE      ; disable interrupts              |B0
        OW_HIZ                  ;                                 |B0
        WAIT    .70             ;                                 |B0
        btfss   GPIO,DQ         ;                                 |B0
        incf    PDByte,F        ;                                 |B0
        bsf     INTCON,GIE      ; enable interrupts               |B0
        WAIT    .430            ;                                 |B0
        return                  ;                                 |B0
;
;  RX byte
;
DSRXBYTE
        bcf     INTCON,GIE      ; disable interrupts              |B0
        movlw   .8              ;                                 |B0
        movwf   Count           ; set bit counter                 |B0
DSRXLP  OW_LO                   ; bring DQ low for 6 usecs        |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        OW_HIZ                  ; change to HiZ and wait 4 usecs  |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        movf    GPIO,W          ; read DQ                         |B0
        andlw   1<<DQ           ; mask off the DQ bit             |B0
        addlw   .255            ; C=1 if DQ=1: C=0 if DQ=0        |B0
        rrf     IOByte,F        ; Shift C into IOBYTE             |B0
        WAIT    .50             ; Wait 50s to end of time slot   |B0
        decfsz  Count,F         ; Decrement the bit counter       |B0
        goto    DSRXLP          ;                                 |B0
        bsf     INTCON,GIE      ; enable interrupts               |B0
        movf    IOByte,W        ;                                 |B0
        return                  ;                                 |B0
;
;  TX byte
;
DSTXBYTE
        bcf     INTCON,GIE      ; disable interrupts              |B0
        movwf   IOByte          ; we send it from IOBYTE          |B0
        movlw   .8              ;                                 |B0
        movwf   Count           ; set bit counter                 |B0
DSTXLP  OW_LO                   ; drive the line low for 3us      |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        nop                     ;                                 |B0
        rrf     IOByte,F        ;                                 |B0
        bsf     STATUS,RP0      ; bank 1                          |B1
        btfsc   STATUS,C        ; test LSB of IOBYTE for 1 or 0   |B1
        bsf     TRISIO,DQ       ; HiZ the line if bit is 1        |B1
        bcf     STATUS,RP0      ; bank 0                          |B0
        WAIT    .60             ; continue driving line for 60s  |B0
        OW_HIZ                  ; release the line for pullup     |B0
        nop                     ;                                 |B0
        nop                     ; recovery time of 2s            |B0
        decfsz  Count,F         ; decrement the bit counter       |B0
        goto    DSTXLP          ;                                 |B0
        bsf     INTCON,GIE      ; enable interrupts               |B0
        return                  ;                                 |B0

;******************************************************************
;
;  get temperature reading
;
GetTemp
        call    OW_RESET        ; send reset signal               |B0
        btfss   PDByte,0        ; DS18B20 found?                  |B0
        return                  ; no, return, else                |B0
        movlw   h'CC'           ;                                 |B0
        call    DSTXBYTE        ; send 'skip rom' command         |B0
        movlw   h'44'           ;                                 |B0
        call    DSTXBYTE        ; start temperature conversion    |B0
ReadSlot
        bsf     INTCON,GIE      ; enable interrupts               |B0
        WAIT    .60             ; wait 60-usecs                   |B0
        bcf     INTCON,GIE      ; disable interrupts              |B0
        OW_LO                   ; start new read slot             |B0
        OW_HIZ                  ;                                 |B0
        nop                     ;
        nop                     ;
        nop                     ;
        nop                     ;
        nop                     ;
        btfss   GPIO,DQ         ; conversion complete '1'?        |B0
        goto    ReadSlot        ; no, do another slot, else       |B0
        bsf     INTCON,GIE      ; enable interrupts               |B0
        WAIT    .60             ; finish read slot period         |B0
GetResult
        call    OW_RESET        ; send reset signal               |B0
        movlw   h'CC'           ;                                 |B0
        call    DSTXBYTE        ; send 'skip rom' command         |B0
        movlw   h'BE'           ;                                 |B0
        call    DSTXBYTE        ; send 'read scratchpad' command  |B0
        call    DSRXBYTE        ;                                 |B0
        movwf   TempL           ; save temperature lo             |B0
        call    DSRXBYTE        ;                                 |B0
        movwf   TempH           ; save temperature hi             |B0
        return                  ;                                 |B0

;******************************************************************
;
;  BC2DC - convert binary C*16 to decimal C*10 in D3:D0 vars
;
;  formula is: C*10 = (10*(Celsius*16))/16
;                    = (10*(TempH:TempL))/16
;
;  D3:D0 result '-250' to '1250' (C*10)
;
BC2DC
        movlw   d'10'           ; multiplier 10                   |B0
        call    Multiply        ; do 10 x TempH:TempL             |B0
        goto    Div16           ;                                 |B0

;******************************************************************
;
;  BC2DF - convert binary C*16 to decimal F*10 in D3:D0 vars
;
;  formula is: F*10 = (18*(Celsius*16)+(320*16))/16
;                    = (18*(TempH:TempL)+( 5120))/16
;
;  D3:D0 result '-670' to '2570' (F*10)
;
BC2DF
        movlw   d'18'           ; multiplier 18 (9/5*10)          |B0
        call    Multiply        ; do 18 x TempH:TempL             |B0
;
;  add (320 * 16) to result
;
        movlw   low (.320*.16)  ; add (320 * 16) to Product       |B0
        addwf   ProdL,F         ;                                 |B0
        skpnc                   ;                                 |B0
        incf    ProdH,F         ;                                 |B0
        movlw   high (.320*.16) ;                                 |B0
        addwf   ProdH,F         ;                                 |B0
        skpnc                   ;                                 |B0
        incf    ProdU,F         ;                                 |B0
;
;  divide result by 16
;
Div16
        movlw   d'4'            ;                                 |B0
        movwf   Count           ;                                 |B0
DivNext
        clrc                    ; clear C                         |B0
        rrf     ProdH,F         ;                                 |B0
        rrf     ProdL,F         ;                                 |B0
        decfsz  Count,F         ; all done?                       |B0
        goto    DivNext         ; no, branch, else                |B0
;
;  check for negative number result
;
        bcf     TempH,7         ; clear "negative" flag           |B0
        btfss   ProdH,3         ; negative result?                |B0
        goto    Bin2Dec         ; no, branch, else                |B0
;
;  convert it to a postive number
;
        bsf     TempH,7         ; set "negative" flag             |B0
        movf    ProdL,W         ; temperature low                 |B0
        sublw   h'00'           ; two's complement it             |B0
        movwf   ProdL           ; ^ is that a verb (grin)?        |B0
        movf    ProdH,W         ; temperature high in W           |B0
        skpc                    ; borrow?  no, skip, else         |B0
        incf    ProdH,W         ; temperature high + 1 in W       |B0
        sublw   h'00'           ; two's complement it             |B0
        movwf   ProdH           ;                                 |B0
;
;  Peter Hemsley's 12-bit Bin_to_BCD (ProdH:ProdL binary input
;  and D3:D0 unpacked BCD output)
;
Bin2Dec
        movf    ProdH,w         ; 16-bit binary hi                |B0
        iorlw   0xF0            ; w=H2-16                         |B0
        movwf   D1              ; D1=H2-16                        |B0
        addwf   D1,f            ; D1=H2*2-32                      |B0
        addwf   D1,f            ; D1=H2*3-48                      |B0
        movwf   D2              ; D2=H2-16                        |B0
        addlw   -D'5'           ; w=H2-21                         |B0
        addwf   D2,f            ; D2=H2*2-37 Done!                |B0
        addlw   D'41'           ; w=H2+20                         |B0
        movwf   D0              ; D0=H2+20                        |B0
        swapf   ProdL,w         ; 16-bit binary lo                |B0
        iorlw   0xF0            ; w=H1-16                         |B0
        addwf   D1,f            ; D1=H2*3+H1-64                   |B0
        addwf   D0,f            ; D0=H2+H1+4, C=1                 |B0
        rlf     D0,f            ; D0=(H2+H1)*2+9, C=0             |B0
        comf    D0,f            ; D0=-(H2+H1)*2-10                |B0
        rlf     D0,f            ; D0=-(H2+H1)*4-20                |B0
        movf    ProdL,w         ;                                 |B0
        andlw   0x0F            ; w=H0                            |B0
        addwf   D0,f            ; D0=H0-(H2+H1)*4-20 Done!        |B0
        rlf     D1,f            ; C=0, D1=H2*6+H1*2-128 Done!     |B0
        movlw   D'5'            ;                                 |B0
        movwf   D3              ;                                 |B0
        movlw   D'10'           ;                                 |B0
mod0
        addwf   D0,f            ; D(X)=D(X)mod10                  |B0
        decf    D1,f            ; D(X+1)=D(X+1)+D(X)div10         |B0
        skpc                    ;                                 |B0
        goto    mod0            ;                                 |B0
mod1
        addwf   D1,f            ;                                 |B0
        decf    D2,f            ;                                 |B0
        skpc                    ;                                 |B0
        goto    mod1            ;                                 |B0
mod2
        addwf   D2,f            ;                                 |B0
        decf    D3,f            ;                                 |B0
        skpc                    ;                                 |B0
        goto    mod2            ;                                 |B0
        return                  ;                                 |B0

;******************************************************************
;
Multiply
        movwf   Mult8           ; set multiplier                  |B0
        movf    TempL,W         ; raw (Celsius * 16) lo           |B0
;       movlw   low  (h'FEE3')  ; -17.8125C (-0.1F) sim test
        movwf   MultL           ; set multiplicand lo             |B0
        movf    TempH,W         ; raw (Celsius * 16) hi           |B0
;       movlw   high (h'FEE3')  ; -17.8125C (-0.1F) sim test
        movwf   MultH           ; set multiplicand hi             |B0
Mult8x16
        clrf    ProdL           ; clear Product registers         |B0
        clrf    ProdH           ;                                 |B0
        clrf    ProdU           ;                                 |B0
        movlw   .8              ;                                 |B0
        movwf   Count           ; 8 bit multiplier                |B0
MultLoop
        clrc                    ; clear Carry                     |B0
        rlf     ProdL,f         ; multiply result (x2)            |B0
        rlf     ProdH,f         ;                                 |B0
        rlf     ProdU,f         ;                                 |B0
        btfss   Mult8,7         ;                                 |B0
        goto    MultNext        ;                                 |B0
        movf    MultL,W         ; get multiplicand lo             |B0
        addwf   ProdL,F         ; add to product lo               |B0
        skpnc                   ; carry?  no, skip, else          |B0
        incf    ProdH,F         ;                                 |B0
        movf    MultH,W         ; get multiplicand hi             |B0
        addwf   ProdH,F         ; add to product hi               |B0
        skpnc                   ; carry?  no, skip, else          |B0
        incf    ProdU,F         ;
MultNext
        rlf     Mult8,F         ; rotate multiplier               |B0
        decfsz  Count,f         ; done?                           |B0
        goto    MultLoop        ; no, branch, else                |B0
        return                  ;                                 |B0

;******************************************************************
;*                                                                *
;*                                                                *
;*                                                                *
;*                                                                *
;*                                                                *
;******************************************************************

Main    bsf     STATUS,RP0      ; bank 1                          |B1
        call    h'3FF'          ; get factory calibration value   |B1
        movwf   OSCCAL          ; set OSCCAL calibration value    |B1
        clrf    ANSEL           ; set all pins to digital mode    |B1
        movlw   b'00011111'     ; GP5/TXPIN output                |B1
        movwf   TRISIO          ; all others inputs               |B1
        bcf     STATUS,RP0      ; bank 0                          |B0
        movlw   b'00111111'     ; set all outputs high            |B0
        movwf   GPIO            ;                                 |B0
;
;  initialize/turn on RS232 package
;
        call    Init232         ;                                 |B0
;
;  _Title macro - home cursor, clear screen, and print a string
;
        _Title  "K8LH 12F675 DS18B20 Demo v2.0\r\n"
;
;  get a fresh Celsius temperature reading (-55C..125C)
;
;  The resulting TempH:TempL word is a 16-bit sign-extended two's 
;  complement number with a value of -55*16 to 125*16 (0.0625C
;  resolution)
;
Test
        call    GetTemp         ;                                 |B0
        Print   "\r\nTemp: "    ; start a new line                |B0

;
;  convert the 12-bit Celsius binary data to decimal C (*10)
;
        call    BC2DC           ; bin Celsius to dec Celsius      |B0
        call    SendResults     ; send C results                 |B0
        Print   "C "            ;                                 |B0
;
;  convert the 12-bit Celsius binary data to decimal F (*10)
;
        call    BC2DF           ; bin Celsius to dec Fahrenheit   |B0
        call    SendResults     ; send F results                 |B0
        Print   "F "            ;                                 |B0
;
;  wait for keyboard entry from Hyperterminal
;
Loop    call    Get232          ; receive character               |B0
        goto    Test

;******************************************************************
;
SendResults
        movf    D3,W            ; the "Hundreds" digit            |B0
        skpnz                   ; leading '0'?  no, skip, else    |B0
        movlw   a' '^'0'        ; use a space character           |B0
        xorlw   a'0'            ;                                 |B0
        btfsc   TempH,7         ; neg number?  no, skip, else     |B0
        movlw   a'-'            ; use minus sign instead          |B0
        call    Put232          ; send it out the RS-232 port     |B0
        movf    D2,W            ; the "Tens" digit                |B0
        iorlw   a'0'            ;                                 |B0
        call    Put232          ;                                 |B0
        movf    D1,W            ; the "Ones" digit                |B0
        iorlw   a'0'            ;                                 |B0
        call    Put232          ;                                 |B0
        movlw   a'.'            ;                                 |B0
        call    Put232          ;                                 |B0
        movf    D0,W            ; the "Tenths" digit              |B0
        iorlw   a'0'            ;                                 |B0
        goto    Put232          ;                                 |B0

	end

